@@ -207,6 +207,7 @@ Json::Value CollationInformationExports(cmGeneratorTarget const* gt)
 {
   Json::Value tdi_exports = Json::arrayValue;
   std::string export_name = gt->GetExportName();
+  std::string fs_export_name = gt->GetFilesystemExportName();
 
   auto const& all_install_exports = gt->GetGlobalGenerator()->GetExportSets();
   for (auto const& exp : all_install_exports) {
@@ -232,6 +233,7 @@ Json::Value CollationInformationExports(cmGeneratorTarget const* gt)
 
       tdi_export_info["namespace"] = ns;
       tdi_export_info["export-name"] = export_name;
+      tdi_export_info["filesystem-export-name"] = fs_export_name;
       tdi_export_info["destination"] = dest;
       tdi_export_info["cxx-module-info-dir"] = cxxm_dir;
       tdi_export_info["export-prefix"] = export_prefix;
@@ -241,8 +243,10 @@ Json::Value CollationInformationExports(cmGeneratorTarget const* gt)
     }
   }
 
-  auto const& all_build_exports = gt->Makefile->GetExportBuildFileGenerators();
-  for (auto const& exp : all_build_exports) {
+  auto const& all_build_exports =
+    gt->GetGlobalGenerator()->GetBuildExportSets();
+  for (auto const& exp_entry : all_build_exports) {
+    auto const* exp = exp_entry.second;
     std::vector<cmExportBuildFileGenerator::TargetExport> targets;
     exp->GetTargets(targets);
 
@@ -268,6 +272,7 @@ Json::Value CollationInformationExports(cmGeneratorTarget const* gt)
 
     tdi_export_info["namespace"] = ns;
     tdi_export_info["export-name"] = export_name;
+    tdi_export_info["filesystem-export-name"] = fs_export_name;
     tdi_export_info["destination"] = dest;
     tdi_export_info["cxx-module-info-dir"] = cxxm_dir;
     tdi_export_info["export-prefix"] = export_prefix;
@@ -315,6 +320,7 @@ struct CxxModuleBmiInstall
 struct CxxModuleExport
 {
   std::string Name;
+  std::string FilesystemName;
   std::string Destination;
   std::string Prefix;
   std::string CxxModuleInfoDir;
@@ -352,6 +358,7 @@ cmDyndepCollation::ParseExportInfo(Json::Value const& tdi)
       CxxModuleExport exp;
       exp.Install = tdi_export["install"].asBool();
       exp.Name = tdi_export["export-name"].asString();
+      exp.FilesystemName = tdi_export["filesystem-export-name"].asString();
       exp.Destination = tdi_export["destination"].asString();
       exp.Prefix = tdi_export["export-prefix"].asString();
       exp.CxxModuleInfoDir = tdi_export["cxx-module-info-dir"].asString();
@@ -426,8 +433,9 @@ bool cmDyndepCollation::WriteDyndepMetadata(
 
     std::string const export_dir =
       cmStrCat(exp.Prefix, '/', exp.CxxModuleInfoDir, '/');
-    std::string const property_file_path = cmStrCat(
-      export_dir, "target-", exp.Name, '-', export_info.Config, ".cmake");
+    std::string const property_file_path =
+      cmStrCat(export_dir, "target-", exp.FilesystemName, '-',
+               export_info.Config, ".cmake");
     properties = cm::make_unique<cmGeneratedFileStream>(property_file_path);
 
     // Set up the preamble.
