@@ -3,7 +3,7 @@ __declspec(dllimport)
 #endif
   extern void bar();
 
-int main()
+int main(void)
 {
   bar();
   return 0;
